/// ReWindowLibrary
/// A C++20 Library for creating and managing windows in a platform-independent manner
/// Developed and Maintained by the boys @ Redacted Software.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file keyboard.h
/// @desc A class that models the functionality of a keyboard device.
/// @edit 2024-07-29

#pragma once

#include <rewindow/types/inputdevice.hpp>
#include "rewindow/data/scancode.h"
#include <Event.h>
#include <map>

namespace ReWindow
{
    using Key = ReScanCode;
    using KeyState = BinaryActuatorState;
    struct KeyboardState
    {
        std::map<Key, KeyState> Keys;
    };

    class KeyPressedEvent : public InputEvent {};
    class KeyReleasedEvent : public InputEvent {};

    class Keyboard : public InputDevice
    {
    public:
        Keyboard() {}
        ~Keyboard() {}
        bool IsKeyDown() const;
        KeyState GetKeyState() const;

        void Update(double elapsed) override;
        DeviceType GetType() const override { return DeviceType::KEYBOARD;}
    public:
        static Event<const KeyPressedEvent&> KeyPressed;
        static Event<const KeyReleasedEvent&> KeyReleased;

    public:
        static Keyboard* GetCurrentDevice();
        static bool GetKeyState(const Key& k);
        static KeyboardState GetState();
    };
}