#include <rewindow/types/mouse.h>

namespace ReWindow
{
    namespace detail
    {
        static Mouse* internal_mouse;

        void UpdateMouse(Mouse* mouse) {
            detail::internal_mouse = mouse;
        }

    }

    namespace InputService
    {
        namespace Mouse
        {
            Vector2 GetMousePosition()
            {
                return detail::internal_mouse->GetPosition();
            }

            bool IsButtonDown(const MouseButton &btn) {
                return detail::internal_mouse->IsButtonDown(btn);
            }
        }

    }


    void Mouse::DoButtonPress(const MouseButton &btn) {
        SetButtonPressed(btn, true);
    }

    void Mouse::DoButtonRelease(const MouseButton &btn) {
        SetButtonPressed(btn, false);
    }

    DeviceType Mouse::GetType() const { return DeviceType::MOUSE;}

    void Mouse::Update(double elapsed) {
        current_state.Left.TimeHeld += elapsed;
        current_state.Right.TimeHeld += elapsed;
        current_state.Middle.TimeHeld += elapsed;
        current_state.Mouse4.TimeHeld += elapsed;
        current_state.Mouse5.TimeHeld += elapsed;

        previous_state = current_state;
    }

    ButtonState Mouse::GetButtonState(const MouseButton &btn) const {
        if (btn == MouseButtons::Left)   return current_state.Left;
        if (btn == MouseButtons::Right)  return current_state.Right;
        if (btn == MouseButtons::Middle) return current_state.Middle;
        if (btn == MouseButtons::Mouse4) return current_state.Mouse4;
        if (btn == MouseButtons::Mouse5) return current_state.Mouse5;
    }

    bool Mouse::IsButtonDown(const MouseButton &btn) const {
        return GetButtonState(btn).Pressed;
    }

    Mouse::~Mouse() {}

    Mouse::Mouse() {}

    void Mouse::SetButtonPressed(const MouseButton &btn, bool pressed) {
        auto state = GetButtonState(btn);
        state.Pressed = pressed;
        state.TimeHeld = 0;
        SetButtonState(btn, state);
    }

    void Mouse::SetButtonState(const MouseButton &btn, ButtonState state) {
        if (btn == MouseButtons::Left)   current_state.Left = state;
        if (btn == MouseButtons::Right)  current_state.Right = state;
        if (btn == MouseButtons::Middle) current_state.Middle = state;
        if (btn == MouseButtons::Mouse4) current_state.Mouse4 = state;
        if (btn == MouseButtons::Mouse5) current_state.Mouse5 = state;
    }
}